/*
 * A queue mechanism for concurrent access to a resource.
 *
 * Copyright (C) 2019, 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/sys/types.h>

#include <pthread.h>
#include <deque>

#include <ipc/message.h>

/* Employed interfaces. */

#include <fsserver/queue_interface.h>



/* A queue abstraction. */

class QueueObject : public Queue
{
  /* A queue of threads and the current running thread. */

  std::deque<l4_cap_idx_t> _queue;
  l4_cap_idx_t _running = L4_INVALID_CAP;

  /* The endpoint for interacting with the queue and its own thread. */

  l4_cap_idx_t _thread_ep;
  pthread_t _thread;

  /* Message handling. */

  ipc_message_t _msg;
  Queue *_client = 0;

protected:
  /* Maintenance and scheduling methods. */

  virtual void remove();

  virtual void next();

  virtual void new_member();

  l4_cap_idx_t pop();

public:
  explicit QueueObject()
  {
  }

  Queue *client();

  /* Initiation, dispatch and finalisation methods. */

  virtual long start();

  virtual void mainloop();

  virtual long end();

  /* Queue operation methods. */

  virtual long join(l4_cap_idx_t user);

  virtual long join_first(l4_cap_idx_t user);

  virtual long leave();

  virtual long yield();
};
