/*
 * Filesystem object opening functionality.
 *
 * Copyright (C) 2018, 2019, 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include "opener_support.h"
#include "resource_server.h"



/* Obtain the accessor and resource server for a filesystem object. */

long OpenerSupport::open_object(fs_object_t *fsobj, int flags,
                                size_t *size, l4_cap_idx_t *file)
{
  Resource *resource;

  /* Initialise a server configuration with a resource. */

  long err = open_object_resource(fsobj, flags, &resource);

  if (err)
    return err;

  /* Complete the initialisation and start a server in a new thread. */

  ResourceServer server(resource);
  err = server.start_thread();
  
  /* Return the file size. */
  /* Return the server capability to the caller. */
  
  *size = fsobj->size;
  *file = server.config()->server;
  return err;
}
